# > \brief \b LSAME
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       LOGICAL FUNCTION lsame(CA,CB)
#
#       .. Scalar Arguments ..
#       CHARACTER CA,CB
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# > LSAME returns .TRUE. if CA is the same letter as CB regardless of
# > case.
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] CA
# > \verbatim
# >          CA is CHARACTER*1
# > \endverbatim
# >
# > \param[in] CB
# > \verbatim
# >          CB is CHARACTER*1
# >          CA and CB specify the single characters to be compared.
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date December 2016
#
# > \ingroup aux_blas
#
#  =====================================================================
def lsame(CA, CB):
    return CA.lower() == CBlower()


#
#  -- Reference BLAS level1 routine (version 3.1) --
#  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
#  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
#     December 2016
#
#     .. Scalar Arguments ..
#   CHARACTER CA,CB
#     ..
#
# =====================================================================
#
#     .. Intrinsic Functions ..
#   INTRINSIC ICHAR
#     ..
#     .. Local Scalars ..
#   INTEGER INTA,INTB,ZCODE
#     ..
#
#     Test if the characters are equal
#
#   LSAME = CA == CB
#   if (LSAME) return
#
#     Now test for equivalence if both characters are alphabetic.
#
#   ZCODE = ICHAR('Z')
#
#     Use 'Z' rather than 'A' so that ASCII can be detected on Prime
#     machines, on which ICHAR returns a value with bit 8 set.
#     ICHAR('A') on Prime machines returns 193 which is the same as
#     ICHAR('A') on an EBCDIC machine.
#
#   INTA = ICHAR(CA)
#   INTB = ICHAR(CB)
#
#   if (ZCODE==90  or  ZCODE==122):
#
#        ASCII is assumed - ZCODE is the ASCII code of either lower or
#        upper case 'Z'.
#
#   if (INTA>=97  and  INTA<=122) INTA = INTA - 32
#   if (INTB>=97  and  INTB<=122) INTB = INTB - 32
#
#   elif (ZCODE==233  or  ZCODE==169):
#
#        EBCDIC is assumed - ZCODE is the EBCDIC code of either lower or
#        upper case 'Z'.
#
#       if (INTA>=129  and  INTA<=137  or
#  +        INTA>=145  and  INTA<=153  or
#  +        INTA>=162  and  INTA<=169) INTA = INTA + 64
#       if (INTB>=129  and  INTB<=137  or
#  +        INTB>=145  and  INTB<=153  or
#  +        INTB>=162  and  INTB<=169) INTB = INTB + 64
#
#   elif (ZCODE==218  or  ZCODE==250):
#
#        ASCII is assumed, on Prime machines - ZCODE is the ASCII code
#        plus 128 of either lower or upper case 'Z'.
#
#       if (INTA>=225  and  INTA<=250) INTA = INTA - 32
#       if (INTB>=225  and  INTB<=250) INTB = INTB - 32
#   END IF
#   LSAME = INTA == INTB
#
#     return
#
#     End of LSAME
#
#   END



def lsame(a, b):
    return a.lower() == b.lower()


def sign(x):
    return 1 if x >= 0 else -1


def slice_(N, inc):
    if inc > 0:
        return slice(None, N * inc, inc)
    else:
        return slice(-(N - 1) * inc, None, inc)



# > \brief \b XERBLA
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def XERBLA( SRNAME, INFO )
#
#       .. Scalar Arguments ..
#       CHARACTER*(*)      SRNAME
#       INTEGER            INFO
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# > XERBLA  is an error handler for the LAPACK routines.
# > It is called by an LAPACK routine if an input parameter has an
# > invalid value.  A message is printed and execution stops.
# >
# > Installers may consider modifying the STOP statement in order to
# > call system-specific exception-handling facilities.
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] SRNAME
# > \verbatim
# >          SRNAME is CHARACTER*(*)
# >          The name of the routine which called XERBLA.
# > \endverbatim
# >
# > \param[in] INFO
# > \verbatim
# >          INFO is INTEGER
# >          The position of the invalid parameter in the parameter list
# >          of the calling routine.
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date December 2016
#
# > \ingroup aux_blas
#
#  =====================================================================
def xerbla(srname, info):
    #
    #  -- Reference BLAS level1 routine (version 3.7.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     December 2016
    #
    #     .. Scalar Arguments ..
    # CHARACTER*(*)      SRNAME
    # INTEGER            INFO
    #     ..
    #
    # =====================================================================
    #
    #     .. Intrinsic Functions ..
    # INTRINSIC          LEN_TRIM
    #     ..
    #     .. Executable Statements ..
    #
    raise Exception(
        " ** On entry to "
        + srname
        + " parameter number "
        + info
        + " had an illegal value"
    )
    # WRITE( *, FMT = 9999 ) SRNAME( 1:LEN_TRIM( SRNAME ) ), INFO


#
# STOP
#
#  9999 FORMAT( ' ** On entry to ', A, ' parameter number ', I2, ' had ', 'an illegal value' )
#
#     End of XERBLA
#
# END



# > \brief \b XERBLA_ARRAY
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def XERBLA_ARRAY(SRNAME_ARRAY, SRNAME_LEN, INFO)
#
#       .. Scalar Arguments ..
#       INTEGER SRNAME_LEN, INFO
#       ..
#       .. Array Arguments ..
#       CHARACTER(1) SRNAME_ARRAY(SRNAME_LEN)
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# > XERBLA_ARRAY assists other languages in calling XERBLA, the LAPACK
# > and BLAS error handler.  Rather than taking a Fortran string argument
# > as the function's name, XERBLA_ARRAY takes an array of single
# > characters along with the array's length.  XERBLA_ARRAY then copies
# > up to 32 characters of that array into a Fortran string and passes
# > that to XERBLA.  If called with a non-positive SRNAME_LEN,
# > XERBLA_ARRAY will call XERBLA with a string of all blank characters.
# >
# > Say some macro or other device makes XERBLA_ARRAY available to C99
# > by a name lapack_xerbla and with a common Fortran calling convention.
# > Then a C99 program could invoke XERBLA via:
# >    {
# >      int flen = strlen(__func__);
# >      lapack_xerbla(__func__, &flen, &info);
# >    }
# >
# > Providing XERBLA_ARRAY is not necessary for intercepting LAPACK
# > errors.  XERBLA_ARRAY calls XERBLA.
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] SRNAME_ARRAY
# > \verbatim
# >          SRNAME_ARRAY is CHARACTER(1) array, dimension (SRNAME_LEN)
# >          The name of the routine which called XERBLA_ARRAY.
# > \endverbatim
# >
# > \param[in] SRNAME_LEN
# > \verbatim
# >          SRNAME_LEN is INTEGER
# >          The length of the name in SRNAME_ARRAY.
# > \endverbatim
# >
# > \param[in] INFO
# > \verbatim
# >          INFO is INTEGER
# >          The position of the invalid parameter in the parameter list
# >          of the calling routine.
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date December 2016
#
# > \ingroup aux_blas
#
#  =====================================================================
from xerbla import xerbla


def XERBLA_ARRAY(SRNAME_ARRAY, SRNAME_LEN, INFO):
    #
    #  -- Reference BLAS level1 routine (version 3.7.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     December 2016
    #
    #     .. Scalar Arguments ..
    # INTEGER SRNAME_LEN, INFO
    #     ..
    #     .. Array Arguments ..
    # CHARACTER(1) SRNAME_ARRAY(SRNAME_LEN)
    #     ..
    #
    # =====================================================================

    SRNAME = ""
    for I in range(SRNAME_LEN, len(SRNAME)):
        SRNAME += SRNAME_ARRAY[I]

    xerbla(SRNAME, INFO)
